import net.mikekohn.java_grinder.CPU;
import net.mikekohn.java_grinder.IOPort0;
import net.mikekohn.java_grinder.Timer;
import net.mikekohn.java_grinder.TimerListener;

// 16,000,000 cycles * 0.000050 = 800 cycles
// 800 cycles = 50us
// 20 interrupts = 1ms
// 40 interrupts = 2ms
// 400 interrupts = 20ms
// 20,000 interrupts = 1s

public class SingingFish implements TimerListener
{
  public static final int RELEASED = 20;
  public static final int PULLED = 40;
  public static int interrupt_count = 0;
  public static int servo_count = 0;
  public static int servo0_curr_value = RELEASED;
  public static int servo1_curr_value = RELEASED;
  public static int servo0_new_value = RELEASED;
  public static int servo1_new_value = RELEASED;
  public static int song_number = 0;
  public static int[] script =
  {
     7, 10,
    11, 15,
    16, 20,
    21, 25,
    54, 57,
    59, 62,
    63, 67,
    68, 72,
  };

  public static void waitOneSecond()
  {
    // Spinning for 20,000 interrupts is 1 second.
    // The song file is a little off though.
    interrupt_count = 0;
    while(interrupt_count != 20900);
  }

  public static void waitHalfSecond()
  {
    // Spinning for 20,000 interrupts is 1 second.
    interrupt_count = 0;
    while(interrupt_count != 20000);
  }

  public static void wait200ms()
  {
    // Spinning for 4,000 interrupts is 200 milliseconds.
    interrupt_count = 0;
    while(interrupt_count != 4000);
  }

  public static void audioReset()
  {
    // Pull rest low for 5ms and then leave high 300ms.
    IOPort0.setPinLow(7);

    interrupt_count = 0;
    while(interrupt_count < 100);

    IOPort0.setPinHigh(7);

    // Wait 600ms (docs say 300ms, but 300ms doesn't work).  Could be
    // a clock inaccuracy on the MSP430.
    interrupt_count = 0;
    while(interrupt_count < 12000);
  }

  public static void sendWord(int data)
  {
    int bits;

    // Wait 300ms.
    //interrupt_count = 0;
    //while(interrupt_count < 6000);

    // Pull clock low for 2ms.
    IOPort0.setPinLow(1);

    interrupt_count = 0;
    while(interrupt_count < 40);

    // Clock out 16 bits.
    for(bits = 0; bits < 16; bits++)
    {
      // Set data line to most significant bit.
      if ((data & 0x8000) == 0)
      {
        IOPort0.setPinLow(0);
      }
        else
      {
        IOPort0.setPinHigh(0);
      }

      data = data << 1;

      // Set clock low for 200us / high 200us
      IOPort0.setPinLow(1);

      interrupt_count = 0;
      while(interrupt_count <= 4);

      IOPort0.setPinHigh(1);

      interrupt_count = 0;
      while(interrupt_count <= 4);
    }

    //IOPort0.setPinHigh(1);
  }

  public static void playSong()
  {
    // Send reset signal.
    audioReset();

    // Sending this 16 bit number representing which song to play.
    sendWord(song_number);

    // Sending command 0xfffe to play song.
    sendWord(0xfffe);
  }

  public static void waitForPlay()
  {
    int buttons;

    while(true)
    {
      buttons = IOPort0.getPortInputValue();

      if ((buttons & 0x40) == 0) { return; }

      if ((buttons & 0x20) == 0)
      {
        // Select button was pressed.
        song_number++;
        if (song_number >= 4) { song_number = 0; }

        // Debounce.
        while(true)
        {
          buttons = IOPort0.getPortInputValue();
          wait200ms();
          if ((buttons & 0x20) != 0) { break; }
        }
      }
    }
  }

  public static void moveMotors()
  {
    int second = 0;
    int wait;
    int n, i;

    for (n = 0; n < script.length; n++)
    {
      wait = script[n] - second;

      for (i = 0; i < wait; i++)
      {
        waitOneSecond();

        second++;

        if ((second & 0xf) == 8)
        {
          servo1_new_value = PULLED;
        }
          else
        {
          servo1_new_value = RELEASED;
        }
      }

      if ((n & 1) == 0)
      {
        servo0_new_value = PULLED;
      }
        else
      {
        servo0_new_value = RELEASED;
      }
    }

    servo0_new_value = RELEASED;
    servo1_new_value = RELEASED;
  }

  public static void main(String args)
  {
    int count;

    CPU.setClock16();

    // P1.0 = Data  (Audio board)
    // P1.1 = Clock (Audio board)
    // P1.2 = Servo 0
    // P1.3 = Servo 1
    // P1.5 = Select Button
    // P1.6 = Play Button
    // P1.7 = Reset (Audio board)
    IOPort0.setPinsAsOutput(0x8f);
    IOPort0.setPinsLow(0xff);
    IOPort0.setPinsHigh(0xe2);
    IOPort0.setPinsResistorEnable(0x60);

    // Setup timer interrupt (aka, Java listener).
    Timer.setInterval(800, 1);
    Timer.setListener(true);

    // Set servo values.
    servo0_new_value = RELEASED;
    servo1_new_value = RELEASED;

    while(true)
    {
      // Poll buttons.
      waitForPlay();

      // Send commands to start song.
      playSong();

      // Move the motors while the song plays
      moveMotors();
    }
  }

  public void timerInterrupt()
  {
    interrupt_count++;
    servo_count++;

    // After 20ms, reset servo pulse (turn off since transistors reverse
    // logic).  Set servo pulses to the values for the next 20ms cycle.
    if (servo_count == 400)
    {
      IOPort0.setPinsLow(0x0c);
      servo_count = 0;
      servo0_curr_value = servo0_new_value;
      servo1_curr_value = servo1_new_value;
    }

    // Set servo pulse to 0 (reverse logic because of transistor).
    if (servo0_curr_value == servo_count) { IOPort0.setPinsHigh(0x04); }
    if (servo1_curr_value == servo_count) { IOPort0.setPinsHigh(0x08); }
  }
}


